#!/usr/bin/env python3
import argparse
import os
import subprocess

## TODO time to print, job priority, mail on completion
## TODO input slot, resolution, OutputBin
## TODO lprm/cancel, lpq, lpstat, lpmove

parser = argparse.ArgumentParser(description="Skript zum Drucken in der Fachschaft.", epilog="See also: lprm, cancel, lpmove, lpq, lpstat")
parser.add_argument("-n", "-#", "--number", type=int, default=1, help="Anzahl der Kopien (default: %(default)s)")
parser.add_argument("file", nargs='*', help="zu druckendes Dokument")
group = parser.add_mutually_exclusive_group()
group.add_argument("-d", "--duplex", action="store_true", help="Duplexfunktion explizit aktivieren")
group.add_argument("--short-edge", action="store_true", help="Duplexfunktion (kurze Seite) explizit aktivieren")
group.add_argument("-s", "--single", action="store_true", help="Duplexfunktion explizit deaktivieren")
group = parser.add_mutually_exclusive_group()
group.add_argument("-H", "--hoern", action="store_true", help="Wähle den Kopierer in Raum 2014.")
group.add_argument("-K", "--karman", action="store_true", help="Wähle den Kopierer in der Karmanstraße. (derzeit default)")
group.add_argument("-O", "--other", action="store_true", help="Wähle den alten Drucker in Raum 2015.")
group.add_argument("-C", "--cluster", action="store_true", help="Wähle den S/W-Druckercluster auf der Hörn.")
parser.add_argument("-p", "--pages", help="Drucke nur bestimmte Seiten, z.B. 1-4,7")
parser.add_argument("--remove", action="store_true", help="Lösche die Datei(n) nach dem Druck.")
parser.add_argument("--landscape", action="store_true", help="Im Querformat drucken.")
group = parser.add_mutually_exclusive_group()
group.add_argument("--fitplot", action="store_true", help="Datei auf Seitengröße skalieren.")
group.add_argument("--scaling", type=int, help="Datei auf n Prozent der Seite skalieren.")
parser.add_argument("--staple", choices=['None', 'Center', 'UpperLeft', 'UpperRight'], help="Tackern. (Karman-only)")
parser.add_argument("--punch", choices=['None', '2Hole', '2HoleEUR', '3Hole', '4Hole', '2', '3', '4'], default='None', help="Lochen. (Karman-only)")
parser.add_argument("--pages-per-page", choices=['1', '2', '4', '6', '9', '16'], default='1', help="Drucke n Seiten auf eine echte Seite. (default: %(default)s)")
parser.add_argument("--media-type", choices=['Auto', 'Highqlty', 'Prepunched', 'Transparency', 'Bond', 'Labels', 'PrnDef', 'Cardstock', 'Letterhead', 'Recycled', 'Color', 'Plain', 'Rough', 'Envelope', 'Preprinted', 'Thick', 'Vellum'], help="Wähle das eingelegte Druckerpapier aus.")
parser.add_argument("--color", choices=['CMYK', 'Gray'], help="Wähle die Farbeinstellungen aus.")
group = parser.add_mutually_exclusive_group()
group.add_argument("--size", choices=['A3', 'A4', 'A5', 'A6'], help="Größe der Seiten festlegen. (A3 Karman-only)")
group.add_argument("--custom-size", help="Benutzerdefinierte Größe der Seiten angeben (<Breite>x<Höhe>)")
parser.add_argument("--title", help="Setze den Titel/Namen des Auftrags.")
parser.add_argument("--options", help="zusätzliche Optionen an den Drucker geben, vergleiche lpr -o")
parser.add_argument("-v", "--verbose", action="store_true", help="Ausführliche Ausgabe aktivieren.")
parser.add_argument("--dry-run", action="store_true", help="Den Druckbefehl nicht tatsächlich ausführen. (Debgging und so.)")
args = parser.parse_args()

if type(args.file) == None or args.file == [] or args.file == "":
    parser.error("Keine zu druckende Datei angegeben.")
if type(args.file) == str:
    args.file = [args.file]

for f in args.file:
    if not os.path.exists(f) or not os.path.isfile(f):
        parser.error(f+" existiert nicht oder ist keine Datei.")

command = ["lpr", "-H", "printsrv.fsmpi.rwth-aachen.de:631", "-#", str(args.number)]
if args.duplex:
    command += ["-o", "Duplex=DuplexNoTumble", "-o", "sides=two-sided-long-edge"]
if args.short_edge:
    command += ["-o", "Duplex=DuplexTumble", "-o", "sides=two-sided-short-edge"]
if args.single:
    command += ["-o", "Duplex=None", "-o", "sides=one-sided"]
if args.hoern:
    command += ["-P", "Hoern_Kopierer"]
    if not args.duplex and not args.short_edge and not args.single:
        command += ["-o", "Duplex=DuplexNoTumble", "-o", "sides=two-sided-long-edge"]
if args.karman:
    command += ["-P", "Kopierer"]
if args.other:
    command += ["-P", "Raum_Hoern"]
if args.cluster:
    command += ["-P", "Cluster_Hoern"]
if args.pages:
    command += ["-o", "page-ranges="+args.pages]
if args.remove:
    command += ["-r"]
if args.title:
    command += ["-T", args.title]
if args.fitplot:
    command += ["-o", "fitplot"]
if args.scaling:
    command += ["-o", "scaling="+str(args.scaling)]
if args.staple:
    if args.hoern or args.other:
        if args.staple != 'None':
            parser.error("Die Drucker auf der Hörn können nicht tackern.")
    elif not args.karman:
        command += ["-P", "Kopierer"]
        args.karman = True
    if args.karman:
        if args.staple == "UpperLeft":
            command += ["-o", "KCStaple=Upperleft"]
        else:
            command += ["-o", "KCStaple="+args.staple]
if args.punch:
    if args.hoern or args.other:
        if args.punch != 'None':
            parser.error("Die Drucker auf der Hörn können nicht lochen.")
    elif not args.karman:
        command += ["-P", "Kopierer"]
        args.karman = True
    if args.karman:
        punch_option = args.punch
        if len(punch_option) == 1:
            punch_option += "Hole"
            if args.punch == "2":
                punch_option += "EUR"
        command += ["-o", "KCPunch=" + punch_option]
if args.size:
    if (args.hoern or args.other) and args.size == 'A3':
        parser.error("Die Drucker auf der Hörn können nicht in A3 drucken.")
    elif args.size == 'A3' and not args.karman:
        command += ["-P", "Kopierer"]
        args.karman = True
    command += ["-o", "PageSize="+args.size]
    if args.size == 'A4':
        command += ["-o", "media=a4"]
if args.custom_size:
    print("Beachte: Benutzerdefinierte Größenangaben werden derzeit nicht überprüft, sondern direkt an CUPS/den Drucker weitergegeben.")
    command += ["-o", "PageSize=Custom."+args.custom_size]
if args.media_type:
    command += ["-o", "MediaType="+args.media_type]
if args.color:
    command += ["-o", "ColorModel="+args.color]
if args.landscape:
    command += ["-o", "landscape"]
if args.pages_per_page and args.pages_per_page != '1':
    command += ["-o", "number-up="+args.pages_per_page]
if args.options:
    command += ["-o", args.options]
for f in args.file:
    command += [f]
if args.verbose:
    print(command)
    if args.dry_run:
        pass
    else:
        subprocess.call(command)
else:
    # subprocess.DEVNULL was only introduced in 3.3, *sigh*
    with open(os.devnull, 'w') as fp:
        if args.dry_run:
            pass
        else:
            subprocess.call(command, stderr=fp, stdout=fp)
